<?php
namespace AkTest\App\Config;

use Ak\App\Config\ConfigFactory;
use Laminas\ConfigAggregator\ArrayProvider;
use Laminas\ConfigAggregator\ConfigAggregator;
use PHPUnit\Framework\TestCase;

/**
 * Class ConfigFactoryTest
 *
 * @package AkTest\App\Config
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/autokreso/app-config
 */
final class ConfigFactoryTest extends TestCase
{
    protected function setUp(): void
    {
        parent::setUp();
        $this->removeFiles();
    }


    protected function tearDown(): void
    {
        parent::tearDown();
        $this->removeFiles();
    }

    private function removeFiles()
    {
        if (file_exists(__DIR__ . '/data/cached.php')) {
            unlink(__DIR__ . '/data/cached.php');
        }
    }

    public function testWillPassTheDevelopmentFlag()
    {
        $factory = new ConfigFactory([
            'development' => true,
        ]);

        $config = $factory();

        $this->assertArrayHasKey('development', $config);
        $this->assertEquals(true, $config['development']);

        $this->assertArrayHasKey(ConfigAggregator::ENABLE_CACHE, $config);
        $this->assertEquals(false, $config[ConfigAggregator::ENABLE_CACHE]);
    }

    public function testWillCreateVarIfDoesntExist()
    {
        $cwd = getcwd();
        chdir(__DIR__);

        $factory = new ConfigFactory([
            'cache_file' => 'var/cached.php',
        ]);

        $factory();

        $this->assertDirectoryExists(__DIR__ . '/var');

        if (is_dir(__DIR__ . '/var')) {
            if (file_exists(__DIR__ . '/var/cached.php')) {
                unlink(__DIR__ . '/var/cached.php');
            }

            rmdir(__DIR__ . '/var');
        }

        chdir($cwd);
    }

    public function testWillCacheMergedConfig()
    {
        $factory = new ConfigFactory([
            'cache_file' => __DIR__ . '/data/cached.php',
        ]);

        $config = $factory([
            new ArrayProvider([
                'some_key' => 'value',
            ]),
        ]);

        $this->assertFileExists(__DIR__ . '/data/cached.php');
        /** @noinspection PhpIncludeInspection */
        $cached = include __DIR__ . '/data/cached.php';

        $this->assertEquals($config, $cached);
    }

    public function testWillLoadAutoloadedConfig()
    {
        $factory = new ConfigFactory([
            'autoload_dir' => __DIR__ . '/data/autoload',
        ]);

        $config = $factory();

        $this->assertArrayHasKey('first', $config);
        $this->assertArrayHasKey('second', $config);
    }

    public function testAutoloadLocalWillOverrideGlobal()
    {
        $factory = new ConfigFactory([
            'autoload_dir' => __DIR__ . '/data/autoload',
        ]);

        $config = $factory();

        $this->assertEquals('overriden', $config['first']['setting']);
    }
}
